/*
  Large integer division algorithm for (days + msecs).

  04/2019 : F. Prill, DWD.

  The method uses a basis b=1200 representation for time spans and the
  division algorithm explained in

  P. B. Hansen: Multiple-Length Division Revisited: A Tour of the Minefield (1992)
                Electrical Engineering and Computer Science Technical Reports. 166.
*/

#ifndef INT_DIV_H_
#define INT_DIV_H_

#include <stdint.h>

/* array representation has (w+1) digits: */
#define W_DIGITS 6

typedef int64_t t_int;

/* data type for a natural number represented by an array of
   (W_DIGITS+1) digits in radix b. */
typedef t_int t_number[W_DIGITS + 1];


void         number_init(t_number *x, const t_int val, const unsigned int m);
void         print_number(const t_number x);
unsigned int length(const t_number x);
void         product(t_number *x, const t_number y, const t_int k, const t_int b, int* ret);
void         quotient(t_number *x, const t_number y, t_int k, const t_int b);
void         iremainder(t_number *x, const t_number y, t_int k, const t_int b);
t_int        trial(const t_number r, const t_number d, const int k, const int m, const t_int b, int* ret);
int          smaller(const t_number r, const t_number dq, const int k, const int m);
void         difference(t_number *r, const t_number dq, const int k, const int m, const t_int b, int *ret);
int          division(const t_number x, const t_number y, t_number *q, t_number *r, const t_int b);

void         convert_basis(const t_int v_in, t_number *v_out, const int digits, const int istart);
t_int        convert_to_decimal(const t_number v_in, const int istart, const int iend);

int          divide_timespan(const t_int days1, const t_int day_fraction1,
                             const t_int days2, const t_int day_fraction2,
                             t_int *q_decimal, t_int *remainder_days, t_int *remainder_ms);

#endif /* INT_DIV_H_ */
