  ! ================================================================================
  ! event section:
  ! ================================================================================

  ! generic assignment for constructors
  !
  FUNCTION t_event_assign_raw(name, referenceDate, firstdate, lastDate, interval, offset)
    TYPE(t_event) :: t_event_assign_raw
    CHARACTER(len=*), INTENT(in)           :: name
    CHARACTER(len=*), INTENT(in)           :: referenceDate
    CHARACTER(len=*), INTENT(in)           :: firstDate
    CHARACTER(len=*), INTENT(in)           :: lastDate
    CHARACTER(len=*), INTENT(in)           :: interval
    CHARACTER(len=*), INTENT(in), OPTIONAL :: offset

    CHARACTER(len=4)                       :: zeroOffset = "PT0S"

    t_event_assign_raw%eventName              = name
    t_event_assign_raw%eventReferenceDateTime = t_datetime(referenceDate)
    t_event_assign_raw%eventFirstDateTime     = t_datetime(firstDate)
    t_event_assign_raw%eventLastDateTime      = t_datetime(lastDate)
    t_event_assign_raw%eventInterval          = t_timedelta(interval)
    IF (PRESENT(offset)) THEN
      t_event_assign_raw%eventOffset = t_timedelta(offset)
    ELSE
      t_event_assign_raw%eventOffset = t_timedelta(zeroOffset)
    ENDIF

    ALLOCATE(t_event_assign_raw%triggerNextEventDateTime)
    t_event_assign_raw%triggerNextEventDateTime = t_event_assign_raw%getEventFirstTriggerDateTime()

    t_event_assign_raw%neverTriggerEvent   = .FALSE.
    t_event_assign_raw%triggerCurrentEvent = .FALSE.
    t_event_assign_raw%eventisFirstInDay   = .FALSE.
    t_event_assign_raw%eventisFirstInMonth = .FALSE.
    t_event_assign_raw%eventisFirstInYear  = .FALSE.
    t_event_assign_raw%eventisLastInDay    = .FALSE.
    t_event_assign_raw%eventisLastInMonth  = .FALSE.
    t_event_assign_raw%eventisLastInYear   = .FALSE.
    t_event_assign_raw%nextEventIsFirst    = .TRUE.
    t_event_assign_raw%lastEventWasFinal   = .FALSE.

  END FUNCTION t_event_assign_raw

  FUNCTION t_event_assign_types(name, referenceDate, firstdate, lastDate, interval, offset)
    TYPE(t_event) :: t_event_assign_types
    CHARACTER(len=*), INTENT(in)            :: name
    TYPE(t_datetime), INTENT(in)            :: referenceDate
    TYPE(t_datetime), INTENT(in)            :: firstDate
    TYPE(t_datetime), INTENT(in)            :: lastDate
    TYPE(t_timedelta), INTENT(in)           :: interval
    TYPE(t_timedelta), INTENT(in), OPTIONAL :: offset

    CHARACTER(len=4)                       :: zeroOffset = "PT0S"

    t_event_assign_types%eventName              = name
    t_event_assign_types%eventReferenceDateTime = referenceDate
    t_event_assign_types%eventFirstDateTime     = firstDate
    t_event_assign_types%eventLastDateTime      = lastDate
    t_event_assign_types%eventInterval          = interval
    IF (PRESENT(offset)) THEN
      t_event_assign_types%eventOffset = offset
    ELSE
      t_event_assign_types%eventOffset = t_timedelta(zeroOffset)
    ENDIF

    ALLOCATE(t_event_assign_types%triggerNextEventDateTime)
    t_event_assign_types%triggerNextEventDateTime = t_event_assign_types%getEventFirstTriggerDateTime()

    t_event_assign_types%neverTriggerEvent   = .FALSE.
    t_event_assign_types%triggerCurrentEvent = .FALSE.
    t_event_assign_types%eventisFirstInDay   = .FALSE.
    t_event_assign_types%eventisFirstInMonth = .FALSE.
    t_event_assign_types%eventisFirstInYear  = .FALSE.
    t_event_assign_types%eventisLastInDay    = .FALSE.
    t_event_assign_types%eventisLastInMonth  = .FALSE.
    t_event_assign_types%eventisLastInYear   = .FALSE.
    t_event_assign_types%nextEventIsFirst    = .TRUE.
    t_event_assign_types%lastEventWasFinal   = .FALSE.

  END FUNCTION t_event_assign_types


  FUNCTION t_event_getEventFirstTriggerDateTime(this)  RESULT(result_dt)
    CLASS(t_event), INTENT(IN) :: this
    TYPE(t_datetime)           :: result_dt
    !
    TYPE(c_ptr) :: c_pointer,tmp_evFirstDateTime, tmp_evOffset, tmp_evInterval, &
      &            tmp_evReferenceDateTime
    TYPE(datetime),  POINTER :: tmp_dt

    tmp_evFirstDateTime            = this%eventFirstDateTime%get_c_pointer()
    tmp_evOffset                   = this%eventOffset%get_c_pointer()
    tmp_evInterval                 = this%eventInterval%get_c_pointer()
    tmp_evReferenceDateTime        = this%eventReferenceDateTime%get_c_pointer()

    c_pointer = this%eventFirstDateTime%get_c_pointer()
    c_pointer = my_geteventfirsttriggerdatetime(tmp_evFirstDateTime, tmp_evInterval, tmp_evOffset, &
      &                                         tmp_evReferenceDateTime, c_pointer)
    CALL C_F_POINTER(c_pointer, tmp_dt)
    result_dt%dt = tmp_dt

    IF (C_ASSOCIATED(c_pointer))                      CALL my_deallocatedatetime(c_pointer)
    IF (C_ASSOCIATED(tmp_evFirstDateTime))            CALL my_deallocatedatetime(tmp_evFirstDateTime)
    IF (C_ASSOCIATED(tmp_evOffset))                   CALL my_deallocatetimedelta(tmp_evOffset)
    IF (C_ASSOCIATED(tmp_evInterval))                 CALL my_deallocatetimedelta(tmp_evInterval)
    IF (C_ASSOCIATED(tmp_evReferenceDateTime))        CALL my_deallocatedatetime(tmp_evReferenceDateTime)

  END FUNCTION t_event_getEventFirstTriggerDateTime


  ! Iterate to next event in event group.
  !
  ! @return NULL() if no next event available.
  FUNCTION t_event_next_event(this)
    TYPE(t_event), POINTER :: t_event_next_event
    CLASS(t_event) :: this
    t_event_next_event => NULL()
    IF (ASSOCIATED(this%nextEventInGroup)) THEN
      t_event_next_event => this%nextEventInGroup
    ENDIF
  END FUNCTION t_event_next_event

  FUNCTION t_event_getId(this) RESULT(res)
    INTEGER(c_int64_t) :: res
    CLASS (t_event) :: this
    res = this%eventId
  END FUNCTION t_event_getId

  FUNCTION t_event_getName(this) RESULT(res)
    CHARACTER(len=max_event_str_len) :: res
    CLASS (t_event) :: this
    res = this%eventName
  END FUNCTION t_event_getName

  FUNCTION t_event_getFirstDatetime (this) RESULT(res)
    TYPE(t_datetime)        :: res
    CLASS(t_event)          :: this
    res = this%eventFirstDateTime
  END FUNCTION t_event_getFirstDatetime

  FUNCTION t_event_getInterval(this) RESULT(res)
    TYPE(t_timedelta)        :: res
    CLASS(t_event)           :: this
    res = this%eventInterval
  END FUNCTION t_event_getInterval

  FUNCTION t_event_getLastDatetime(this) RESULT(res)
    TYPE(t_datetime)        :: res
    CLASS(t_event)          :: this
    res = this%eventLastDateTime
  END FUNCTION t_event_getLastDatetime

  FUNCTION t_event_getNextOccurrenceDatetime(this, query_start_dt) RESULT(res)
    TYPE(t_datetime)             :: res
    CLASS(t_event)               :: this
    TYPE(t_datetime), INTENT(IN), OPTIONAL :: query_start_dt
    !
    TYPE(datetime),  POINTER :: dt_tmp
    TYPE(c_ptr)              :: tmp_dt, tmp_evLastDateTime, tmp_evInterval
    TYPE(c_ptr)              :: event_triggerNextEventDateTime, c_pointer1
    LOGICAL(c_bool)          :: event_nextEventIsFirst

    res = this%triggerNextEventDateTime

    IF (PRESENT(query_start_dt)) THEN
      ! if a start for our query is given, invoke
      ! "my_gettriggernexteventatdatetime":

      ! --- copy-in:

      c_pointer1 = res%get_c_pointer()	
      tmp_dt                         = query_start_dt%get_c_pointer()
      tmp_evLastDateTime             = this%eventLastDateTime%get_c_pointer()
      tmp_evInterval                 = this%eventInterval%get_c_pointer()
      event_nextEventIsFirst         = this%nextEventIsFirst
      event_triggerNextEventDateTime = c_null_ptr
      IF (ASSOCIATED(this%triggerNextEventDateTime      ))  &
        &   event_triggerNextEventDateTime = this%triggerNextEventDateTime%get_c_pointer()

      ! --- call C routine:

      CALL my_gettriggernexteventatdatetimeraw(                       &
        &             tmp_evLastDateTime, event_nextEventIsFirst,     &
        &             event_triggerNextEventDateTime, tmp_evInterval, &
        &             tmp_dt, c_pointer1)

      ! --- copy-out:

      CALL C_F_POINTER(c_pointer1, dt_tmp)
      res%dt = dt_tmp

      IF (C_ASSOCIATED(event_triggerNextEventDateTime)) THEN
        IF (.NOT. ASSOCIATED(this%triggerNextEventDateTime)) ALLOCATE(this%triggerNextEventDateTime)
        CALL C_F_POINTER(event_triggerNextEventDateTime, dt_tmp)
        this%triggerNextEventDateTime%dt = dt_tmp
      END IF

      ! --- clean up:

      CALL my_deallocatedatetime(tmp_dt)
      IF (C_ASSOCIATED(tmp_evLastDateTime))  CALL my_deallocatedatetime(tmp_evLastDateTime)
      IF (C_ASSOCIATED(tmp_evInterval))      CALL my_deallocatetimedelta(tmp_evInterval)

      IF (C_ASSOCIATED(event_triggerNextEventDateTime      )) &
        &     CALL my_deallocatedatetime(event_triggerNextEventDateTime      )

      CALL my_deallocatedatetime(c_pointer1)
    END IF
  END FUNCTION t_event_getNextOccurrenceDatetime

  FUNCTION t_event_getPrevOccurrenceDatetime(this) RESULT(res)
    TYPE(t_datetime)             :: res
    CLASS(t_event)               :: this
    res = this%triggeredPreviousEventDateTime
  END FUNCTION t_event_getPrevOccurrenceDatetime


  FUNCTION t_event_is_active(this, my_datetime, plus_slack, minus_slack) result(ret)
    CLASS(t_event), TARGET      :: this
    TYPE(t_datetime)            :: my_datetime
    TYPE(t_timedelta), OPTIONAL :: plus_slack
    TYPE(t_timedelta), OPTIONAL :: minus_slack
    TYPE(c_ptr)     :: tmp_dt, tmp_plus, tmp_minus, tmp_evFirstDateTime, tmp_evLastDateTime, tmp_evInterval
    LOGICAL(c_bool) :: event_neverTriggerEvent, event_triggerCurrentEvent, event_eventisFirstInDay, &
                       event_eventisFirstInMonth, event_eventisFirstInYear, event_eventisLastInDay, &
                       event_eventisLastInMonth, event_eventisLastInYear, event_nextEventIsFirst,   &
                       event_lastEventWasFinal, ret
    TYPE(c_ptr)     :: event_triggerNextEventDateTime,     &
                       event_eventsLastEvaluationDateTime, &
                       event_triggeredPreviousEventDateTime
    TYPE(datetime),  POINTER        :: dt_tmp

    ! --- copy-in:

    tmp_dt                  = my_datetime%get_c_pointer()
    tmp_evFirstDateTime     = this%eventFirstDateTime%get_c_pointer()
    tmp_evLastDateTime      = this%eventLastDateTime%get_c_pointer()
    tmp_evInterval          = this%eventInterval%get_c_pointer()

    tmp_plus  = c_null_ptr
    tmp_minus = c_null_ptr
    IF (PRESENT(plus_slack))  tmp_plus  = plus_slack%get_c_pointer()
    IF (PRESENT(minus_slack)) tmp_minus = minus_slack%get_c_pointer()

    event_neverTriggerEvent    = this%neverTriggerEvent
    event_triggerCurrentEvent  = this%triggerCurrentEvent
    event_eventisFirstInDay    = this%eventisFirstInDay
    event_eventisFirstInMonth  = this%eventisFirstInMonth
    event_eventisFirstInYear   = this%eventisFirstInYear
    event_eventisLastInDay     = this%eventisLastInDay
    event_eventisLastInMonth   = this%eventisLastInMonth
    event_eventisLastInYear    = this%eventisLastInYear
    event_nextEventIsFirst     = this%nextEventIsFirst
    event_lastEventWasFinal    = this%lastEventWasFinal

    event_triggerNextEventDateTime       = c_null_ptr
    event_eventsLastEvaluationDateTime   = c_null_ptr
    event_triggeredPreviousEventDateTime = c_null_ptr

    IF (ASSOCIATED(this%triggerNextEventDateTime      ))  &
      &   event_triggerNextEventDateTime       = this%triggerNextEventDateTime%get_c_pointer()
    IF (ASSOCIATED(this%eventsLastEvaluationDateTime  ))  &
      &   event_eventsLastEvaluationDateTime   = this%eventsLastEvaluationDateTime%get_c_pointer()
    IF (ASSOCIATED(this%triggeredPreviousEventDateTime))  &
      &   event_triggeredPreviousEventDateTime = this%triggeredPreviousEventDateTime%get_c_pointer()

    ! --- call C routine:

    ret = my_isCurrentEventActiveRaw(tmp_dt,                               &  ! in
      &                              tmp_evFirstDateTime,                  &  ! in
      &                              tmp_evLastDateTime,                   &  ! in
      &                              tmp_evInterval,                       &  ! in
      &                              tmp_plus,                             &  ! in
      &                              tmp_minus,                            &  ! in
      &                              event_neverTriggerEvent,              &  ! inout
      &                              event_triggerCurrentEvent,            &  ! inout
      &                              event_eventisFirstInDay,              &  ! inout
      &                              event_eventisFirstInMonth,            &  ! inout
      &                              event_eventisFirstInYear,             &  ! inout
      &                              event_eventisLastInDay,               &  ! inout
      &                              event_eventisLastInMonth,             &  ! inout
      &                              event_eventisLastInYear,              &  ! inout
      &                              event_nextEventIsFirst,               &  ! inout
      &                              event_lastEventWasFinal,              &  ! inout
      &                              event_eventsLastEvaluationDateTime,   &  ! inout
      &                              event_triggerNextEventDateTime,       &  ! inout
      &                              event_triggeredPreviousEventDateTime)    ! inout

    ! --- copy-out:

    this%neverTriggerEvent    = event_neverTriggerEvent
    this%triggerCurrentEvent  = event_triggerCurrentEvent
    this%eventisFirstInDay    = event_eventisFirstInDay
    this%eventisFirstInMonth  = event_eventisFirstInMonth
    this%eventisFirstInYear   = event_eventisFirstInYear
    this%eventisLastInDay     = event_eventisLastInDay
    this%eventisLastInMonth   = event_eventisLastInMonth
    this%eventisLastInYear    = event_eventisLastInYear
    this%nextEventIsFirst     = event_nextEventIsFirst
    this%lastEventWasFinal    = event_lastEventWasFinal

    IF (C_ASSOCIATED(event_triggerNextEventDateTime)) THEN
      IF (.NOT. ASSOCIATED(this%triggerNextEventDateTime)) ALLOCATE(this%triggerNextEventDateTime)
      CALL C_F_POINTER(event_triggerNextEventDateTime, dt_tmp)
      this%triggerNextEventDateTime%dt = dt_tmp
    END IF
    IF (C_ASSOCIATED(event_eventsLastEvaluationDateTime)) THEN
      IF (.NOT. ASSOCIATED(this%eventsLastEvaluationDateTime)) ALLOCATE(this%eventsLastEvaluationDateTime)
      CALL C_F_POINTER(event_eventsLastEvaluationDateTime, dt_tmp)
      this%eventsLastEvaluationDateTime%dt = dt_tmp
    END IF
    IF (C_ASSOCIATED(event_triggeredPreviousEventDateTime)) THEN
      IF (.NOT. ASSOCIATED(this%triggeredPreviousEventDateTime)) ALLOCATE(this%triggeredPreviousEventDateTime)
      CALL C_F_POINTER(event_triggeredPreviousEventDateTime, dt_tmp)
      this%triggeredPreviousEventDateTime%dt = dt_tmp
    END IF

    ! --- clean up:

    CALL my_deallocatedatetime(tmp_dt)
    IF (C_ASSOCIATED(tmp_plus))            CALL my_deallocatetimedelta(tmp_plus)
    IF (C_ASSOCIATED(tmp_minus))           CALL my_deallocatetimedelta(tmp_minus)
    IF (C_ASSOCIATED(tmp_evFirstDateTime)) CALL my_deallocatedatetime(tmp_evFirstDateTime)
    IF (C_ASSOCIATED(tmp_evLastDateTime))  CALL my_deallocatedatetime(tmp_evLastDateTime)
    IF (C_ASSOCIATED(tmp_evInterval))      CALL my_deallocatetimedelta(tmp_evInterval)

    IF (C_ASSOCIATED(event_triggerNextEventDateTime      )) &
      &     CALL my_deallocatedatetime(event_triggerNextEventDateTime      )
    IF (C_ASSOCIATED(event_eventsLastEvaluationDateTime  )) &
      &     CALL my_deallocatedatetime(event_eventsLastEvaluationDateTime  )
    IF (C_ASSOCIATED(event_triggeredPreviousEventDateTime)) &
      &     CALL my_deallocatedatetime(event_triggeredPreviousEventDateTime)

  END FUNCTION t_event_is_active

  ! ================================================================================
  ! event group section:
  ! ================================================================================

  FUNCTION t_eventGroup_constructor(name) RESULT(this_event_group)
    TYPE(t_eventGroup) :: this_event_group
    CHARACTER(len=*), INTENT(in) :: name
    event_group_id                        = event_group_id + 1
    this_event_group%event_group_id       = event_group_id
    this_event_group%event_group_name     = name
    this_event_group%first_event_in_group => NULL()
    this_event_group%last_event_in_group  => NULL()
  END FUNCTION t_eventGroup_constructor

  SUBROUTINE t_eventGroup_addToGroup(this, event_to_add)
    CLASS (t_eventGroup) :: this
    TYPE(t_event), TARGET :: event_to_add
    IF (.NOT. ASSOCIATED(this%last_event_in_group)) THEN
      allocate(this%first_event_in_group, source=event_to_add)
      this%last_event_in_group => this%first_event_in_group
      NULLIFY(this%first_event_in_group%nextEventInGroup)
    ELSE
      allocate(this%last_event_in_group%nextEventInGroup, source=event_to_add)
      this%last_event_in_group => this%last_event_in_group%nextEventInGroup
      NULLIFY(this%last_event_in_group%nextEventInGroup)
    ENDIF
  END SUBROUTINE t_eventGroup_addToGroup

  FUNCTION t_eventGroup_getGroupId(this) RESULT(group_id)
    INTEGER :: group_id
    CLASS(t_eventGroup) ::this
    group_id = this%event_group_id
  END FUNCTION t_eventGroup_getGroupId

  FUNCTION t_eventGroup_getGroupName(this) RESULT(name)
    CHARACTER(len=max_groupname_str_len) :: name
    CLASS(t_eventGroup) :: this
    name = this%event_group_name
  END FUNCTION t_eventGroup_getGroupName

  FUNCTION t_eventGroup_getFirstEvent(this) RESULT(event_ptr)
    TYPE(t_event), POINTER :: event_ptr
    CLASS(t_eventGroup) :: this
    event_ptr => this%first_event_in_group
  END FUNCTION t_eventGroup_getFirstEvent
